@@ -26,9 +26,13 @@
 #include <sys/ptrace.h>
 #include <sys/user.h>
 #include <sys/wait.h>
+#include <stdbool.h>
 
 #include "ulp_common.h"
 
+/* System configuration options.  */
+bool check_ptrace_scope(void);
+
 /* Memory read/write helper functions */
 
 int write_bytes_ptrace(const void *buf, size_t n, int pid, Elf64_Addr addr);
